/* tarot implements the rules of the tarot game
 * Copyright (C) 2019  Vivien Kraus
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef H_TAROT_CNN_PRIVATE_IMPL_INCLUDED
#define H_TAROT_CNN_PRIVATE_IMPL_INCLUDED

#include <tarot/features_private.h>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <string.h>
#include <float.h>

static size_t
cnn_predict (const TarotGame * game, size_t max_score, double *score)
{
  double features_noshape[92 * 6 * 32] = { 0 };
  size_t input_dim = features_cnn_add (game, 92 * 6 * 32, features_noshape);
  assert (input_dim == 92 * 6 * 32);
  double first_filters[5][5][5][15] =
#include "first_filters.h"
    ;
  double first_filters_out[5][32][6][92];
  size_t i, j, k, l;
  size_t full_index;
  double features[32][6][92];
  full_index = 0;
  for (i = 0; i < 32; i++)
    {
      for (j = 0; j < 6; j++)
        {
          for (k = 0; k < 92; k++)
            {
              features[i][j][k] = features_noshape[full_index++];
            }
        }
    }
  for (i = 0; i < 5; i++)
    {
      for (j = 0; j < 32; j++)
        {
          for (k = 0; k < 6; k++)
            {
              for (l = 0; l < 92; l++)
                {
                  int m, n, o;
                  double out = 0;
                  for (m = -2; m <= 2; m++)
                    {
                      for (n = -2; n <= 2; n++)
                        {
                          for (o = -7; o <= 7; o++)
                            {
                              int i1 = j + m;
                              int i2 = k + n;
                              int i3 = l + o;
                              if (i1 >= 0 && i2 >= 0 && i3 >= 0
                                  && i1 < 32 && i2 < 6 && i3 < 92)
                                {
                                  out +=
                                    features[i1][i2][i3] *
                                    first_filters[i][m + 2][n + 2][o + 7];
                                }
                            }
                        }
                    }
                  if (out >= 0)
                    {
                      first_filters_out[i][j][k][l] = out;
                    }
                  else
                    {
                      first_filters_out[i][j][k][l] = 0;
                    }
                }
            }
        }
    }
  double first_max_pooling_out[5][8][3][24];
  for (i = 0; i < 5; i++)
    {
      for (j = 0; j < 8; j++)
        {
          for (k = 0; k < 3; k++)
            {
              for (l = 0; l < 24; l++)
                {
                  size_t m, n, o;
                  double out = 0;
                  for (m = 0; m < 4; m++)
                    {
                      for (n = 0; n < 2; n++)
                        {
                          for (o = 0; o < 4; o++)
                            {
                              size_t i1 = j * 4 + m;
                              size_t i2 = k * 2 + n;
                              size_t i3 = l * 4 + o;
                              if (first_filters_out[i][i1][i2][i3] > out)
                                {
                                  out = first_filters_out[i][i1][i2][i3];
                                }
                            }
                        }
                    }
                  first_max_pooling_out[i][j][k][l] = out;
                }
            }
        }
    }
  double second_filters[5][5][3][5] =
#include "second_filters.h"
    ;
  double second_filters_out[5][4][1][20];
  for (i = 0; i < 5; i++)
    {
      for (j = 0; j < 4; j++)
        {
          for (k = 0; k < 1; k++)
            {
              for (l = 0; l < 20; l++)
                {
                  size_t m, n, o, p;
                  double out = 0;
                  for (m = 0; m < 5; m++)
                    {
                      for (n = 0; n < 5; n++)
                        {
                          for (o = 0; o < 3; o++)
                            {
                              for (p = 0; p < 5; p++)
                                {
                                  size_t i2 = j + n;
                                  size_t i3 = k + o;
                                  size_t i4 = l + p;
                                  double filter_value =
                                    second_filters[m][n][o][p];
                                  out +=
                                    (first_max_pooling_out[m][i2][i3][i4]
                                     * filter_value);
                                }
                            }
                        }
                    }
                  if (out >= 0)
                    {
                      second_filters_out[i][j][k][l] = out;
                    }
                  else
                    {
                      second_filters_out[i][j][k][l] = 0;
                    }
                }
            }
        }
    }
  double second_max_pooling_out[100];
  full_index = 0;
  for (i = 0; i < 5; i++)
    {
      for (j = 0; j < 2; j++)
        {
          for (k = 0; k < 1; k++)
            {
              for (l = 0; l < 10; l++)
                {
                  size_t m, n, o;
                  double out = 0;
                  for (m = 0; m < 2; m++)
                    {
                      for (n = 0; n < 1; n++)
                        {
                          for (o = 0; o < 2; o++)
                            {
                              size_t i1 = j * 2 + m;
                              size_t i2 = k * 1 + n;
                              size_t i3 = l * 2 + o;
                              if (second_filters_out[i][i1][i2][i3] > out)
                                {
                                  out = second_filters_out[i][i1][i2][i3];
                                }
                            }
                        }
                    }
                  second_max_pooling_out[full_index++] = out;
                }
            }
        }
    }
  double fc[6][100] =
#include "fully_connected.h"
    ;
  double final_output[6];
  for (i = 0; i < 6; i++)
    {
      double out = 0;
      for (j = 0; j < 100; j++)
        {
          out += fc[i][j] * second_max_pooling_out[j];
        }
      final_output[i] = out;
    }
  size_t n_players = tarot_game_n_players (game);
  for (i = 0; i < max_score && i < n_players; i++)
    {
      score[i] = final_output[i];
    }
  return n_players;
}

static inline size_t
cnn_predictor_construct (size_t max_mem, char *mem, size_t *alignment)
{
  size_t ret = predictor_construct (max_mem, mem, alignment);
  if (max_mem >= ret)
    {
      TarotPredictor *predictor = (TarotPredictor *) mem;
      predictor->cnn = 1;
    }
  return ret;
}

#include <tarot/features_private_impl.h>

#endif /* H_TAROT_CNN_PRIVATE_IMPL_INCLUDED */
