/* tarot implements the rules of the tarot game
 * Copyright (C) 2017, 2018, 2019  Vivien Kraus
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>
#include "gettext.h"
#include <tarot/features_private.h>
#include "xalloc.h"
#include "gettext.h"
#define _(String) dgettext (PACKAGE, String)

static int libtarot_initialized = 0;

char *tarot_libtarot_program_name = PACKAGE_NAME;

static char *libtarot_localedir = NULL;
static char *libtarot_datadir = NULL;
static char *libtarot_pkgdatadir = NULL;

int tarot_libtarot_with_guile = HAVE_GUILE;

static void
tarot_set_localedir (const char *localedir)
{
  free (libtarot_localedir);
  libtarot_localedir = strdup (localedir);
}

void
tarot_set_datadir (const char *datadir)
{
  free (libtarot_datadir);
  free (libtarot_pkgdatadir);
  libtarot_datadir = strdup (datadir);
  libtarot_pkgdatadir = xmalloc (strlen (libtarot_datadir)
                                 + strlen ("/") + strlen (PACKAGE) + 1);
  strcpy (libtarot_pkgdatadir, libtarot_datadir);
  strcat (libtarot_pkgdatadir, "/");
  strcat (libtarot_pkgdatadir, PACKAGE);
}

int
tarot_init (const char *localedir)
{
  assert (libtarot_initialized == 0);
  tarot_set_localedir (localedir);
  bindtextdomain (PACKAGE, libtarot_localedir);
  _tarot_features_init (libtarot_pkgdatadir);
  libtarot_initialized = 1;
  return 0;
}

void
tarot_quit (void)
{
  libtarot_initialized = 0;
}

const char *
tarot_libtarot_version (void)
{
  static const char *effective_version = LIBTAROT_VERSION_INFO;
  return effective_version;
}

void
tarot_mcts_set_parameter (TarotMcts * mcts, double value)
{
  (void) mcts;
  (void) value;
}

double
tarot_mcts_get_parameter (const TarotMcts * mcts)
{
  (void) mcts;
  return 0;
}

void
tarot_mcts_set_simulation_random (TarotMcts * mcts, double value)
{
  (void) mcts;
  (void) value;
}

double
tarot_mcts_get_simulation_random (const TarotMcts * mcts)
{
  (void) mcts;
  return 0;
}

void
tarot_mcts_set_simulation_agreed (TarotMcts * mcts, double value)
{
  (void) mcts;
  (void) value;
}

double
tarot_mcts_get_simulation_agreed (const TarotMcts * mcts)
{
  (void) mcts;
  return 0;
}

size_t
tarot_mcts_default_iterations ()
{
  return 100;
}

const struct TarotMctsNode *
tarot_mcts_root (const TarotMcts * mcts)
{
  (void) mcts;
  return NULL;
}

const TarotGameEvent *
tarot_mcts_get (const TarotMctsNode * node, size_t *n_simulations,
                int *sum_scores)
{
  (void) node;
  *n_simulations = 0;
  *sum_scores = 0;
  return NULL;
}

const TarotMctsNode *
tarot_mcts_parent (const TarotMctsNode * node)
{
  (void) node;
  return NULL;
}

const TarotMctsNode *
tarot_mcts_first_child (const TarotMctsNode * node)
{
  (void) node;
  return NULL;
}

const TarotMctsNode *
tarot_mcts_next_sibling (const TarotMctsNode * node)
{
  (void) node;
  return NULL;
}

const TarotMctsNode *
tarot_mcts_previous_sibling (const TarotMctsNode * node)
{
  (void) node;
  return NULL;
}

size_t
tarot_mcts_save_to_xml (const TarotMcts * mcts, size_t start, size_t max,
                        char *dest_out)
{
  (void) mcts;
  (void) start;
  if (max != 0)
    {
      dest_out[0] = '\0';
    }
  return 0;
}

char *
tarot_mcts_save_to_xml_alloc (const TarotMcts * mcts)
{
  char *ret = xmalloc (1);
  ret[0] = '\0';
  (void) mcts;
  return ret;
}

#include <tarot/features_private_impl.h>
