/* tarot implements the rules of the tarot game
 * Copyright (C) 2019  Vivien Kraus
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "program_commons.h"

static int encoded = 0;
static void parse_options (int argc, char *argv[]);
static int score[5] = { 0, 0, 0, 0, 0 };

static void
event_handler (const TarotGame * base, TarotPlayer next,
               const TarotGameEvent * event)
{
  char *line =
    tarot_features_arff_data_alloc (base, event, score[next], encoded);
  printf ("%s", line);
  free (line);
}

int
main (int argc, char *argv[])
{
  TarotGame *game = NULL;
  size_t n_players;
  tarot_program_setup ();
  parse_options (argc, argv);
  game = tarot_program_read_game ();
  printf ("%s", tarot_features_arff_header (encoded));
  if (tarot_game_get_scores (game, &n_players, 0, 5, score) != TAROT_GAME_OK)
    {
      fprintf (stderr, _("Error: this game is not finished yet.\n"));
      tarot_game_free (game);
      return EXIT_FAILURE;
    }
  assert (n_players <= 5);
  tarot_program_iterate (game, &event_handler);
  tarot_game_free (game);
  return EXIT_SUCCESS;
}

static void
parse_options (int argc, char *argv[])
{
  int c;
  while (1)
    {
      static struct option long_options[] = {
        {N_("help"), no_argument, 0, 'h'},
        {N_("version"), no_argument, 0, 'v'},
        {N_("encoded"), no_argument, 0, 'e'},
        {0, 0, 0, 0}
      };
      int option_index = 0;
      for (option_index = 0; long_options[option_index].name != NULL;
           option_index++)
        {
          long_options[option_index].name =
            _(long_options[option_index].name);
        }
      option_index = 0;
      c = getopt_long (argc, argv, "hve", long_options, &option_index);
      if (c == -1)
        break;
      switch (c)
        {
        case 0:
          break;

        case 'h':
          printf (_("Usage: tarot-features [OPTIONS]... < GAME > FEATURES\n\
Extract the game features.  Options:\n\
- -h, --help: print this message and exit;\n\
- -v, --version: print the program version and exit;\n\
- -e, --encoded: one-hot encode the categorical features.\n\
"));
          exit (EXIT_SUCCESS);
          break;

        case 'v':
          printf (_("%s (libtarot %s)\n\
\n\
Copyright © 2019 Vivien Kraus\nThis is free software; see the source for copying conditions.  There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n"), "tarot-features", tarot_libtarot_version ());
          exit (EXIT_SUCCESS);
          break;

        case 'e':
          encoded = 1;
          break;

        case '?':
          /* getopt_long already printed an error message. */
          exit (EXIT_FAILURE);
          break;

        default:
          abort ();
        }
    }
  if (optind < argc)
    {
      fprintf (stderr, _("Error: extra arguments:\n"));
      while (optind < argc)
        fprintf (stderr, _("- %s\n"), argv[optind++]);
      exit (EXIT_FAILURE);
    }
}
