/* tarot implements the rules of the tarot game
 * Copyright (C) 2019  Vivien Kraus
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "program_commons.h"

int
main (int argc, char *argv[])
{
  TarotGame *game = NULL;
  tarot_program_setup ();
  if (argc != 2)
    {
      fprintf (stderr, _("Usage:\n\
- tarot-status --help\n\
- tarot-status --version\n\
- tarot-status next < GAME\n\
- tarot-status at-end < GAME\n\
- tarot-status score < GAME\n\
- tarot-status errors < GAME\n\
Please run tarot-status --help.\n"));
      return EXIT_FAILURE;
    }
  if (strcmp (argv[1], _("--help")) == 0)
    {
      printf (_("Usage: tarot-status [OPTION]... ([COMMAND] < GAME)?\n\
Print some status information of GAME.  Options are:\n\
- --help: print this message and exit\n\
- --version: print program version and exit\n\
\n\
Commands:\n\
- next: print the number of the next player.  The number of a player\n\
    is the number of players that have made a bid before this player in\n\
    the bidding phase;\n\
- at-end: exit sucessfully if the game is finished, or exit with code\n\
    1 if the game is not finished;\n\
- score: for a finished game, print the score, in the order of player\n\
     numbers, one line for each player;\n\
- errors: print the sum of all errors that all known players made in\n\
     the game.\n"));
      return EXIT_SUCCESS;
    }
  if (strcmp (argv[1], _("--version")) == 0)
    {
      printf (_("%s (libtarot %s)\n\
\n\
Copyright © 2019 Vivien Kraus\nThis is free software; see the source for copying conditions.  There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n"), "tarot-status", tarot_libtarot_version ());
      return EXIT_SUCCESS;
    }
  game = tarot_program_read_game ();
  if (strcmp (argv[1], _("next")) == 0)
    {
      TarotPlayer next;
      if (tarot_game_get_next (game, &next) == TAROT_GAME_OK)
        {
          printf ("%u\n", next);
          tarot_game_free (game);
          return EXIT_SUCCESS;
        }
      else
        {
          fprintf (stderr,
                   _("Error: all players have played in this game.\n"));
          tarot_game_free (game);
          return EXIT_FAILURE;
        }
    }
  if (strcmp (argv[1], _("at-end")) == 0)
    {
      size_t n_scores;
      if (tarot_game_get_scores (game, &n_scores, 0, 0, NULL) ==
          TAROT_GAME_OK)
        {
          tarot_game_free (game);
          return EXIT_SUCCESS;
        }
      tarot_game_free (game);
      return EXIT_FAILURE;
    }
  if (strcmp (argv[1], _("score")) == 0)
    {
      size_t n_scores;
      int i;
      int *scores;
      if (tarot_game_get_scores_alloc (game, &n_scores, &scores) !=
          TAROT_GAME_OK)
        {
          fprintf (stderr, _("Error: this game is not finished yet.\n"));
          tarot_game_free (game);
          return EXIT_FAILURE;
        }
      for (i = 0; i < (int) n_scores; i++)
        {
          printf ("%d\n", scores[i]);
        }
      tarot_game_free (game);
      return EXIT_SUCCESS;
    }
  if (strcmp (argv[1], _("errors")) == 0)
    {
      double ret = tarot_features_errors (game);
      printf ("%f\n", ret);
      tarot_game_free (game);
      return EXIT_SUCCESS;
    }
  fprintf (stderr, _("Error: unknown command '%s'.\n"), argv[1]);
  tarot_game_free (game);
  return EXIT_FAILURE;
}
